@@ -21,9 +21,13 @@
  */
 #include <config.h>
 
-#include <libgnome/libgnome.h>
-
 #include "bc.h"
+
+#include <glib/gi18n.h>
+#include <glib/gmem.h>
+#include <glib/gstrfuncs.h>
+#include <glib/gmessages.h>
+
 #include "bc-postnet.h"
 #include "bc-gnubarcode.h"
 
@@ -168,7 +172,7 @@ id_to_index (const gchar *id)
 		}
 	}
 
-	g_warning( "Unknown barcode id \"%s\"", id );
+	g_message( "Unknown barcode id \"%s\"", id );
 	return 0;
 }
 
@@ -188,7 +192,7 @@ name_to_index (const gchar *name)
 		}
 	}
 
-	g_warning( "Unknown barcode name \"%s\"", name );
+	g_message( "Unknown barcode name \"%s\"", name );
 	return 0;
 }
 
