@@ -21,10 +21,13 @@
  */
 #include <config.h>
 
-#include <glib.h>
+#include "label-object.h"
+
+#include <glib/gmem.h>
+#include <glib/gstrfuncs.h>
+#include <glib/gmessages.h>
 #include <libart_lgpl/libart.h>
 
-#include "label-object.h"
 #include "marshal.h"
 
 #include "debug.h"
@@ -755,7 +758,7 @@ gl_label_object_set_text_line_spacing (glLabelObject     *object,
 /****************************************************************************/
 void
 gl_label_object_set_text_color (glLabelObject     *object,
-				guint              text_color)
+				glColorNode       *text_color_node)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -764,7 +767,7 @@ gl_label_object_set_text_color (glLabelObject     *object,
 	if ( GL_LABEL_OBJECT_GET_CLASS(object)->set_text_color != NULL ) {
 
 		/* We have an object specific method, use it */
-		GL_LABEL_OBJECT_GET_CLASS(object)->set_text_color (object, text_color);
+		GL_LABEL_OBJECT_GET_CLASS(object)->set_text_color (object, text_color_node);
 
 	}
 
@@ -918,10 +921,10 @@ gl_label_object_get_text_line_spacing (glLabelObject     *object)
 /****************************************************************************/
 /* Get text color for all text contained in object.                         */
 /****************************************************************************/
-guint
+glColorNode*
 gl_label_object_get_text_color (glLabelObject     *object)
 {
-	guint ret = 0;
+	glColorNode *ret = NULL;
 
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -966,7 +969,7 @@ gl_label_object_can_fill (glLabelObject     *object)
 /****************************************************************************/
 void
 gl_label_object_set_fill_color (glLabelObject     *object,
-				guint              fill_color)
+				glColorNode       *fill_color_node)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -975,7 +978,7 @@ gl_label_object_set_fill_color (glLabelObject     *object,
 	if ( GL_LABEL_OBJECT_GET_CLASS(object)->set_fill_color != NULL ) {
 
 		/* We have an object specific method, use it */
-		GL_LABEL_OBJECT_GET_CLASS(object)->set_fill_color (object, fill_color);
+		GL_LABEL_OBJECT_GET_CLASS(object)->set_fill_color (object, fill_color_node);
 
 	}
 
@@ -985,10 +988,10 @@ gl_label_object_set_fill_color (glLabelObject     *object,
 /****************************************************************************/
 /* Get fill color for object.                                               */
 /****************************************************************************/
-guint
+glColorNode*
 gl_label_object_get_fill_color (glLabelObject     *object)
 {
-	guint ret = 0;
+	glColorNode *ret = NULL;
 
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -1033,7 +1036,7 @@ gl_label_object_can_line_color (glLabelObject     *object)
 /****************************************************************************/
 void
 gl_label_object_set_line_color (glLabelObject     *object,
-				guint              line_color)
+				glColorNode       *line_color_node)
 {
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -1042,7 +1045,7 @@ gl_label_object_set_line_color (glLabelObject     *object,
 	if ( GL_LABEL_OBJECT_GET_CLASS(object)->set_line_color != NULL ) {
 
 		/* We have an object specific method, use it */
-		GL_LABEL_OBJECT_GET_CLASS(object)->set_line_color (object, line_color);
+		GL_LABEL_OBJECT_GET_CLASS(object)->set_line_color (object, line_color_node);
 
 	}
 
@@ -1052,10 +1055,10 @@ gl_label_object_set_line_color (glLabelObject     *object,
 /****************************************************************************/
 /* Get line color for object.                                               */
 /****************************************************************************/
-guint
+glColorNode*
 gl_label_object_get_line_color (glLabelObject     *object)
 {
-	guint ret = 0;
+	glColorNode *ret = NULL;
 
 	gl_debug (DEBUG_LABEL, "START");
 
@@ -1323,4 +1326,3 @@ merge_changed_cb (glLabel       *label,
 {
 	gl_label_object_emit_changed (object);
 }
-
