@@ -1,4 +1,6 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+
+/**
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
  *  object-editor.c:  object properties editor module
@@ -21,11 +23,17 @@
  */
 #include <config.h>
 
-#include <gnome.h>
+#include "object-editor.h"
+
+#include <glib/gi18n.h>
 #include <glade/glade-xml.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtktogglebutton.h>
+
 #include <math.h>
 
-#include "object-editor.h"
 #include "prefs.h"
 #include "mygal/widget-color-combo.h"
 #include "color.h"
@@ -144,7 +152,7 @@ gl_object_editor_init (glObjectEditor *editor)
 					   NULL);
 
 	if (!editor->priv->gui) {
-		g_warning ("Could not open object-editor.glade, reinstall glabels!");
+		g_critical ("Could not open object-editor.glade. gLabels may not be installed correctly!");
 		return;
 	}
 
@@ -165,6 +173,7 @@ gl_object_editor_init (glObjectEditor *editor)
 
 	/* Hide all notebook pages to start with. */
 	gtk_widget_hide_all (editor->priv->notebook);
+	gtk_widget_set_no_show_all (editor->priv->notebook, TRUE);
 
 	gl_debug (DEBUG_EDITOR, "END");
 }
@@ -183,6 +192,10 @@ gl_object_editor_finalize (GObject *object)
 	g_return_if_fail (GL_IS_OBJECT_EDITOR (editor));
 	g_return_if_fail (editor->priv != NULL);
 
+	if (editor->priv->gui) {
+		g_object_unref (G_OBJECT (editor->priv->gui));
+	}
+
 	g_free (editor->priv);
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT(gl_prefs),
@@ -313,7 +326,7 @@ gl_object_notebook_construct_valist (glObjectEditor       *editor,
 			break;
 
 		default:
-			g_warning ("option = %d", option);
+			g_message ("option = %d", option);
 			g_assert_not_reached ();
 		}
 		
@@ -383,6 +396,18 @@ gl_object_editor_set_key_names (glObjectEditor      *editor,
 					  merge != NULL);
 	}
  
+	if (editor->priv->text_color_key_combo) {
+		gtk_widget_set_sensitive (editor->priv->text_color_key_combo, merge != NULL);
+	}
+ 
+	if (editor->priv->text_color_key_radio) {
+		gtk_widget_set_sensitive (editor->priv->text_color_key_radio, merge != NULL);
+		if (merge == NULL) {
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON(editor->priv->text_color_radio), TRUE);
+		}
+	}	
+	
 	if (editor->priv->edit_insert_field_button) {
 		gtk_widget_set_sensitive (editor->priv->edit_insert_field_button,
 					  merge != NULL);
@@ -411,7 +436,7 @@ gl_object_editor_set_key_names (glObjectEditor      *editor,
 				GTK_TOGGLE_BUTTON(editor->priv->data_literal_radio), TRUE);
 		}
 	}
- 
+	
 	fixed_flag = editor->priv->data_format_fixed_flag;
 	if (editor->priv->data_format_label) {
 		gtk_widget_set_sensitive (editor->priv->data_format_label,
@@ -429,25 +454,81 @@ gl_object_editor_set_key_names (glObjectEditor      *editor,
 		gtk_widget_set_sensitive (editor->priv->data_digits_spin,
 					  (merge != NULL) && !fixed_flag);
 	}
+
+	if (editor->priv->fill_key_combo) {
+		gtk_widget_set_sensitive (editor->priv->fill_key_combo, merge != NULL);
+	}
  
+	if (editor->priv->fill_key_radio) {
+		gtk_widget_set_sensitive (editor->priv->fill_key_radio, merge != NULL);
+		if (merge == NULL) {
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON(editor->priv->fill_color_radio), TRUE);
+		}
+	}
+	
+	if (editor->priv->line_key_combo) {
+		gtk_widget_set_sensitive (editor->priv->line_key_combo, merge != NULL);
+	}
+ 
+	if (editor->priv->line_key_radio) {
+		gtk_widget_set_sensitive (editor->priv->line_key_radio, merge != NULL);
+		if (merge == NULL) {
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON(editor->priv->line_color_radio), TRUE);
+		}
+	}
+	
+	if (editor->priv->bc_key_combo) {
+		gtk_widget_set_sensitive (editor->priv->bc_key_combo, merge != NULL);
+	}
+ 
+	if (editor->priv->bc_key_radio) {
+		gtk_widget_set_sensitive (editor->priv->bc_key_radio, merge != NULL);
+		if (merge == NULL) {
+			gtk_toggle_button_set_active (
+				GTK_TOGGLE_BUTTON(editor->priv->bc_color_radio), TRUE);
+		}
+	}	
+	
         keys = gl_merge_get_key_list (merge);
         if ( keys != NULL ) {
 
 		combo = editor->priv->img_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
 
 		combo = editor->priv->edit_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
 
 		combo = editor->priv->data_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
+		combo = editor->priv->fill_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
 
+		combo = editor->priv->text_color_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+
+		combo = editor->priv->line_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
+		combo = editor->priv->bc_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
                 gl_merge_free_key_list (&keys);
         } else {
 
@@ -455,19 +536,39 @@ gl_object_editor_set_key_names (glObjectEditor      *editor,
 
 		combo = editor->priv->img_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
 
 		combo = editor->priv->edit_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
-
+		
 		combo = editor->priv->data_key_combo;
 		if (combo) {
-			gtk_combo_set_popdown_strings (GTK_COMBO (combo), keys);
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
+		combo = editor->priv->fill_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
+		combo = editor->priv->text_color_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
 		}
 
+		combo = editor->priv->line_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
+		combo = editor->priv->bc_key_combo;
+		if (combo) {
+			gl_util_combo_box_set_strings (GTK_COMBO_BOX (combo), keys);
+		}
+		
                 g_list_free (keys);
         }
  
@@ -565,4 +666,3 @@ prefs_changed_cb (glObjectEditor *editor)
 
 	gl_debug (DEBUG_EDITOR, "END");
 }
-
