@@ -21,12 +21,14 @@
  */
 #include <config.h>
 
+#include "print.h"
+
+#include <glib/gi18n.h>
 #include <math.h>
 #include <time.h>
 #include <ctype.h>
-#include <gtk/gtk.h>
+#include <gtk/gtkenums.h>
 
-#include "print.h"
 #include "label.h"
 #include "label-text.h"
 #include "label-box.h"
@@ -108,13 +110,16 @@ static void       draw_text_object            (PrintInfo        *pi,
 					       glMergeRecord    *record);
 
 static void       draw_box_object             (PrintInfo        *pi,
-					       glLabelBox       *object);
+					       glLabelBox       *object,
+						   glMergeRecord  *record);
 
 static void       draw_line_object            (PrintInfo        *pi,
-					       glLabelLine      *object);
+					       glLabelLine      *object,
+						   glMergeRecord  *record);						   
 
 static void       draw_ellipse_object         (PrintInfo        *pi,
-					       glLabelEllipse   *object);
+					       glLabelEllipse   *object,
+						   glMergeRecord  *record);
 
 static void       draw_image_object           (PrintInfo        *pi,
 					       glLabelImage     *object,
@@ -423,11 +428,11 @@ print_info_new (GnomePrintJob    *job,
 		  "setting page size = \"%s\"", template->page_size);
 
 	gnome_print_config_set_length (pi->config,
-				       GNOME_PRINT_KEY_PAPER_WIDTH,
+				       (guchar *)GNOME_PRINT_KEY_PAPER_WIDTH,
 				       template->page_width,
 				       GNOME_PRINT_PS_UNIT);
 	gnome_print_config_set_length (pi->config,
-				       GNOME_PRINT_KEY_PAPER_HEIGHT,
+				       (guchar *)GNOME_PRINT_KEY_PAPER_HEIGHT,
 				       template->page_height,
 				       GNOME_PRINT_PS_UNIT);
 
@@ -476,7 +481,7 @@ print_page_begin (PrintInfo *pi)
 	pi->sheet++;
 
 	str = g_strdup_printf ("sheet%02d", pi->sheet);
-	gnome_print_beginpage (pi->pc, str);
+	gnome_print_beginpage (pi->pc, (guchar *)str);
 	g_free (str);
 
 	/* Translate and scale, so that our origin is at the upper left. */
@@ -698,11 +703,11 @@ draw_object (PrintInfo     *pi,
 	if (GL_IS_LABEL_TEXT(object)) {
 		draw_text_object (pi, GL_LABEL_TEXT(object), record);
 	} else if (GL_IS_LABEL_BOX(object)) {
-		draw_box_object (pi, GL_LABEL_BOX(object));
+		draw_box_object (pi, GL_LABEL_BOX(object), record);
 	} else if (GL_IS_LABEL_LINE(object)) {
-		draw_line_object (pi, GL_LABEL_LINE(object));
+		draw_line_object (pi, GL_LABEL_LINE(object), record);
 	} else if (GL_IS_LABEL_ELLIPSE(object)) {
-		draw_ellipse_object (pi, GL_LABEL_ELLIPSE(object));
+		draw_ellipse_object (pi, GL_LABEL_ELLIPSE(object), record);
 	} else if (GL_IS_LABEL_IMAGE(object)) {
 		draw_image_object (pi, GL_LABEL_IMAGE(object), record);
 	} else if (GL_IS_LABEL_BARCODE(object)) {
@@ -733,6 +738,7 @@ draw_text_object (PrintInfo     *pi,
 	GnomeFontWeight  font_weight;
 	gboolean         font_italic_flag;
 	guint            color;
+	glColorNode     *color_node;
 	GtkJustification just;
 	gboolean         auto_shrink;
 	GnomeGlyphList  *glyphlist;
@@ -749,7 +755,11 @@ draw_text_object (PrintInfo     *pi,
 	font_size = gl_label_object_get_font_size (GL_LABEL_OBJECT(object));
 	font_weight = gl_label_object_get_font_weight (GL_LABEL_OBJECT(object));
 	font_italic_flag = gl_label_object_get_font_italic_flag (GL_LABEL_OBJECT(object));
-	color = gl_label_object_get_text_color (GL_LABEL_OBJECT(object));
+
+	color_node = gl_label_object_get_text_color (GL_LABEL_OBJECT(object));
+	color = gl_color_node_expand (color_node, record);
+	gl_color_node_free (&color_node);
+	
 	just = gl_label_object_get_text_alignment (GL_LABEL_OBJECT(object));
 	text_line_spacing =
 		gl_label_object_get_text_line_spacing (GL_LABEL_OBJECT(object));
@@ -765,14 +775,16 @@ draw_text_object (PrintInfo     *pi,
 		/* auto shrink text size to keep within text box limits. */
 		for (i = 0; line[i] != NULL; i++) {
 
-			font = gnome_font_find_closest_from_weight_slant (font_family,
-									  font_weight,
-									  font_italic_flag,
-									  font_size);
-			glyphlist = gnome_glyphlist_from_text_dumb (font,
-								    color,
-								    0.0, 0.0,
-								    line[i]);
+			font = gnome_font_find_closest_from_weight_slant (
+				(guchar *)font_family,
+				font_weight,
+				font_italic_flag,
+				font_size);
+			glyphlist = gnome_glyphlist_from_text_dumb (
+				font,
+				color,
+				0.0, 0.0,
+				(guchar *)line[i]);
 			gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 			w = bbox.x1;
 			gnome_glyphlist_unref (glyphlist);
@@ -785,14 +797,15 @@ draw_text_object (PrintInfo     *pi,
 				font_size -= 0.5;
 
 				font = gnome_font_find_closest_from_weight_slant (
-					font_family,
+					(guchar *)font_family,
 					font_weight,
 					font_italic_flag,
 					font_size);
-				glyphlist = gnome_glyphlist_from_text_dumb (font,
-									    color,
-									    0.0, 0.0,
-									    line[i]);
+				glyphlist = gnome_glyphlist_from_text_dumb (
+					font,
+					color,
+					0.0, 0.0,
+					(guchar *)line[i]);
 				gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 				w = bbox.x1;
 				gnome_glyphlist_unref (glyphlist);
@@ -802,7 +815,7 @@ draw_text_object (PrintInfo     *pi,
 	}
 
 	font = gnome_font_find_closest_from_weight_slant (
-                                       font_family,
+                                       (guchar *)font_family,
 				       font_weight,
 				       font_italic_flag,
 				       font_size);
@@ -818,7 +831,7 @@ draw_text_object (PrintInfo     *pi,
 
 		glyphlist = gnome_glyphlist_from_text_dumb (font, color,
 							    0.0, 0.0,
-							    line[i]);
+							    (guchar *)line[i]);
 
 		gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 		w = bbox.x1;
@@ -852,7 +865,7 @@ draw_text_object (PrintInfo     *pi,
 
 		gnome_print_gsave (pi->pc);
 		gnome_print_scale (pi->pc, 1.0, -1.0);
-		gnome_print_show (pi->pc, line[i]);
+		gnome_print_show (pi->pc, (guchar *)line[i]);
 		gnome_print_grestore (pi->pc);
 	}
 
@@ -869,19 +882,28 @@ draw_text_object (PrintInfo     *pi,
 /*---------------------------------------------------------------------------*/
 static void
 draw_box_object (PrintInfo  *pi,
-		 glLabelBox *object)
+		 glLabelBox *object,
+		 glMergeRecord  *record)
 {
-	gdouble w, h;
-	gdouble line_width;
-	guint line_color, fill_color;
+	gdouble      w, h;
+	gdouble      line_width;
+	guint        line_color;
+	glColorNode *line_color_node;
+	glColorNode *fill_color_node;
+	guint        fill_color;
 
 	gl_debug (DEBUG_PRINT, "START");
 
 	gl_label_object_get_size (GL_LABEL_OBJECT(object), &w, &h);
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
-	fill_color = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
-
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	fill_color_node = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
+	line_color = gl_color_node_expand (line_color_node, record);
+	fill_color = gl_color_node_expand (fill_color_node, record);
+	gl_color_node_free (&line_color_node);
+	gl_color_node_free (&fill_color_node);
+	
 	/* Paint fill color */
 	create_rectangle_path (pi->pc, 0.0, 0.0, w, h);
 	gnome_print_setrgbcolor (pi->pc,
@@ -909,17 +931,22 @@ draw_box_object (PrintInfo  *pi,
 /*---------------------------------------------------------------------------*/
 static void
 draw_line_object (PrintInfo   *pi,
-		  glLabelLine *object)
+		  glLabelLine *object,
+		  glMergeRecord  *record)
 {
-	gdouble w, h;
-	gdouble line_width;
-	guint line_color;
+	gdouble      w, h;
+	gdouble      line_width;
+	guint        line_color;
+	glColorNode *line_color_node;
 
 	gl_debug (DEBUG_PRINT, "START");
 
 	gl_label_object_get_size (GL_LABEL_OBJECT(object), &w, &h);
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	line_color = gl_color_node_expand (line_color_node, record);
+	gl_color_node_free (&line_color_node);
 
 	gnome_print_moveto (pi->pc, 0.0, 0.0);
 	gnome_print_lineto (pi->pc, w, h);
@@ -939,18 +966,27 @@ draw_line_object (PrintInfo   *pi,
 /*---------------------------------------------------------------------------*/
 static void
 draw_ellipse_object (PrintInfo      *pi,
-		     glLabelEllipse *object)
+		     glLabelEllipse *object,
+                     glMergeRecord  *record)
 {
-	gdouble x0, y0, rx, ry, w, h;
-	gdouble line_width;
-	guint line_color, fill_color;
+	gdouble      x0, y0, rx, ry, w, h;
+	gdouble      line_width;
+	glColorNode *line_color_node;
+	glColorNode *fill_color_node;
+	guint        line_color;
+	guint        fill_color;
 
 	gl_debug (DEBUG_PRINT, "START");
 
 	gl_label_object_get_size (GL_LABEL_OBJECT(object), &w, &h);
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
-	fill_color = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	fill_color_node = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
+	line_color = gl_color_node_expand (line_color_node, record);
+	fill_color = gl_color_node_expand (fill_color_node, record);
+	gl_color_node_free (&line_color_node);
+	gl_color_node_free (&fill_color_node);
 
 	rx = w / 2.0;
 	ry = h / 2.0;
@@ -1049,6 +1085,7 @@ draw_barcode_object (PrintInfo      *pi,
 	gboolean            text_flag;
 	gboolean            checksum_flag;
 	guint               color;
+	glColorNode        *color_node;
 	guint               format_digits;
 	gdouble             w, h;
 
@@ -1057,7 +1094,11 @@ draw_barcode_object (PrintInfo      *pi,
 	text_node = gl_label_barcode_get_data (object);
 	gl_label_barcode_get_props (object,
 				    &id, &text_flag, &checksum_flag, &format_digits);
-	color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+					
+	color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	color = gl_color_node_expand (color_node, record);
+	gl_color_node_free (&color_node);
+	
 	gl_label_object_get_size (GL_LABEL_OBJECT(object), &w, &h);
 
 	text = gl_text_node_expand (text_node, record);
@@ -1069,9 +1110,9 @@ draw_barcode_object (PrintInfo      *pi,
 	if (gbc == NULL) {
 
 		font = gnome_font_find_closest_from_weight_slant (
-                                               GL_BARCODE_FONT_FAMILY,
-					       GL_BARCODE_FONT_WEIGHT,
-					       FALSE, 12.0);
+			(guchar *)GL_BARCODE_FONT_FAMILY,
+			GL_BARCODE_FONT_WEIGHT,
+			FALSE, 12.0);
 		gnome_print_setfont (pi->pc, font);
 
 		gnome_print_setrgbcolor (pi->pc,
@@ -1086,7 +1127,7 @@ draw_barcode_object (PrintInfo      *pi,
 
 		gnome_print_gsave (pi->pc);
 		gnome_print_scale (pi->pc, 1.0, -1.0);
-		gnome_print_show (pi->pc, _("Invalid barcode data"));
+		gnome_print_show (pi->pc, (guchar *)_("Invalid barcode data"));
 		gnome_print_grestore (pi->pc);
 
 	} else {
@@ -1110,9 +1151,9 @@ draw_barcode_object (PrintInfo      *pi,
 			bchar = (glBarcodeChar *) li->data;
 
 			font = gnome_font_find_closest_from_weight_slant (
-				                       GL_BARCODE_FONT_FAMILY,
-						       GL_BARCODE_FONT_WEIGHT,
-						       FALSE, bchar->fsize);
+				(guchar *)GL_BARCODE_FONT_FAMILY,
+				GL_BARCODE_FONT_WEIGHT,
+				FALSE, bchar->fsize);
 			gnome_print_setfont (pi->pc, font);
 
 			gnome_print_setrgbcolor (pi->pc,
@@ -1130,7 +1171,7 @@ draw_barcode_object (PrintInfo      *pi,
 			cstring = g_strdup_printf ("%c", bchar->c);
 			gnome_print_gsave (pi->pc);
 			gnome_print_scale (pi->pc, 1.0, -1.0);
-			gnome_print_show (pi->pc, cstring);
+			gnome_print_show (pi->pc, (guchar *)cstring);
 			gnome_print_grestore (pi->pc);
 			g_free (cstring);
 
@@ -1207,7 +1248,7 @@ draw_outline (PrintInfo *pi,
 		break;
 
 	default:
-		g_warning ("Unknown template label style");
+		g_message ("Unknown template label style");
 		break;
 	}
 
@@ -1277,7 +1318,7 @@ clip_to_outline (PrintInfo *pi,
 		break;
 
 	default:
-		g_warning ("Unknown template label style");
+		g_message ("Unknown template label style");
 		break;
 	}
 
@@ -1320,7 +1361,7 @@ clip_punchouts (PrintInfo *pi,
 		break;
 
 	default:
-		g_warning ("Unknown template label style");
+		g_message ("Unknown template label style");
 		break;
 	}
 
