@@ -21,11 +21,13 @@
  */
 #include <config.h>
 
-#include <glib.h>
+#include "view-text.h"
 
+#include <glib/gi18n.h>
+#include <glib/gmem.h>
+#include <gtk/gtkmenu.h>
 #include <libgnomeprint/gnome-glyphlist.h>
 
-#include "view-text.h"
 #include "canvas-hacktext.h"
 #include "view-highlight.h"
 
@@ -55,11 +57,12 @@ struct _glViewTextPrivate {
 
 	GList           *item_list;
 
+#ifdef EDIT_TEXT_IN_PLACE
 	GnomeCanvasItem *cursor;
 	gboolean         cursor_visible;
 	gboolean         cursor_state;
 	guint            cursor_timeout;
-
+#endif
 };
 
 /*========================================================*/
@@ -215,7 +218,9 @@ gl_view_text_new (glLabelText *object,
 
 	/* Create analogous canvas item. */
 	draw_hacktext (view_text);
+#ifdef EDIT_TEXT_IN_PLACE
 	draw_cursor (view_text);
+#endif
 
 	g_signal_connect (G_OBJECT (object), "changed",
 			  G_CALLBACK (update_canvas_item_from_object_cb), view_text);
@@ -224,8 +229,10 @@ gl_view_text_new (glLabelText *object,
 			  G_CALLBACK (item_event_cb), view_text);
 
 	buffer = gl_label_text_get_buffer (object);
+#ifdef EDIT_TEXT_IN_PLACE
 	g_signal_connect (G_OBJECT (buffer), "mark-set",
 			  G_CALLBACK (mark_set_cb), view_text);
+#endif
 
 	gl_debug (DEBUG_VIEW, "END");
 
@@ -259,9 +266,9 @@ construct_properties_editor (glViewObject *view_object)
 	gl_object_editor_set_text_buffer (GL_OBJECT_EDITOR(editor), buffer);
 	
 	/* Update */
+	update_editor_from_label_cb (object->parent, GL_OBJECT_EDITOR(editor));
 	update_editor_from_object_cb (object, GL_OBJECT_EDITOR(editor));
 	update_editor_from_move_cb (object, 0, 0, GL_OBJECT_EDITOR(editor));
-	update_editor_from_label_cb (object->parent, GL_OBJECT_EDITOR(editor));
 
 	/* Connect signals. */
 	g_signal_connect (G_OBJECT (editor), "changed",
@@ -273,7 +280,7 @@ construct_properties_editor (glViewObject *view_object)
 	g_signal_connect (G_OBJECT (object), "moved",
 			  G_CALLBACK (update_editor_from_move_cb), editor);
 	g_signal_connect (G_OBJECT (object->parent), "size_changed",
-			  G_CALLBACK (update_editor_from_label_cb), editor);
+			  G_CALLBACK (update_editor_from_label_cb), editor);			  
 	g_signal_connect (G_OBJECT (object->parent), "merge_changed",
 			  G_CALLBACK (update_editor_from_label_cb), editor);
 
@@ -293,7 +300,9 @@ update_canvas_item_from_object_cb (glLabelObject *object,
 
 	/* Adjust appearance of analogous canvas item. */
 	draw_hacktext (view_text);
+#ifdef EDIT_TEXT_IN_PLACE
 	draw_cursor (view_text);
+#endif
 
 	gl_debug (DEBUG_VIEW, "END");
 }
@@ -310,7 +319,7 @@ update_object_from_editor_cb (glObjectEditor *editor,
 	gdouble            font_size;
 	GnomeFontWeight    font_weight;
 	gboolean           font_italic_flag;
-	guint              color;
+	glColorNode       *color_node;
 	GtkJustification   just;
 	gdouble            text_line_spacing;
 	gboolean           auto_shrink;
@@ -329,7 +338,7 @@ update_object_from_editor_cb (glObjectEditor *editor,
 	font_size = gl_object_editor_get_font_size (editor);
 	font_weight = gl_object_editor_get_font_weight (editor);
 	font_italic_flag = gl_object_editor_get_font_italic_flag (editor);
-	color = gl_object_editor_get_text_color (editor);
+	color_node = gl_object_editor_get_text_color (editor);
 	just = gl_object_editor_get_text_alignment (editor);
 	text_line_spacing = gl_object_editor_get_text_line_spacing (editor);
 	auto_shrink = gl_object_editor_get_text_auto_shrink (editor);
@@ -339,11 +348,12 @@ update_object_from_editor_cb (glObjectEditor *editor,
 	gl_label_object_set_font_size (object, font_size);
 	gl_label_object_set_font_weight (object, font_weight);
 	gl_label_object_set_font_italic_flag (object, font_italic_flag);
-	gl_label_object_set_text_color (object, color);
+	gl_label_object_set_text_color (object, color_node);
 	gl_label_object_set_text_alignment (object, just);
 	gl_label_object_set_text_line_spacing (object, text_line_spacing);
 	gl_label_text_set_auto_shrink (GL_LABEL_TEXT (object), auto_shrink);
 
+	gl_color_node_free (&color_node);
 	g_free (font_family);
 
 	g_signal_handlers_unblock_by_func (G_OBJECT(object),
@@ -400,21 +410,23 @@ update_editor_from_object_cb (glLabelObject  *object,
 	gdouble            font_size;
 	GnomeFontWeight    font_weight;
 	gboolean           font_italic_flag;
-	guint              color;
+	glColorNode       *color_node;
 	GtkJustification   just;
 	gdouble            text_line_spacing;
 	gboolean           auto_shrink;
+	glMerge			   *merge;
 
 	gl_debug (DEBUG_VIEW, "START");
 
 	gl_label_object_get_size (object, &w, &h);
 	gl_object_editor_set_size (editor, w, h);
+	merge = gl_label_get_merge (GL_LABEL(object->parent));
 
 	font_family      = gl_label_object_get_font_family (object);
 	font_size        = gl_label_object_get_font_size (object);
 	font_weight      = gl_label_object_get_font_weight (object);
 	font_italic_flag = gl_label_object_get_font_italic_flag (object);
-	color            = gl_label_object_get_text_color (object);
+	color_node       = gl_label_object_get_text_color (object);
 	just             = gl_label_object_get_text_alignment (object);
 	text_line_spacing = gl_label_object_get_text_line_spacing (object);
 	auto_shrink      = gl_label_text_get_auto_shrink (GL_LABEL_TEXT (object));
@@ -423,11 +435,12 @@ update_editor_from_object_cb (glLabelObject  *object,
 	gl_object_editor_set_font_size (editor, font_size);
 	gl_object_editor_set_font_weight (editor, font_weight);
 	gl_object_editor_set_font_italic_flag (editor, font_italic_flag);
-	gl_object_editor_set_text_color (editor, color);
+	gl_object_editor_set_text_color (editor, (merge != NULL), color_node);
 	gl_object_editor_set_text_alignment (editor, just);
 	gl_object_editor_set_text_line_spacing (editor, text_line_spacing);
 	gl_object_editor_set_text_auto_shrink (editor, auto_shrink);
 
+	gl_color_node_free (&color_node);
 	g_free (font_family);
 
 	gl_debug (DEBUG_VIEW, "END");
@@ -491,11 +504,11 @@ gl_view_text_get_create_cursor (void)
 
 	if (!cursor) {
 		pixmap_data = gdk_bitmap_create_from_data (NULL,
-							   cursor_text_bits,
+							   (gchar *)cursor_text_bits,
 							   cursor_text_width,
 							   cursor_text_height);
 		pixmap_mask = gdk_bitmap_create_from_data (NULL,
-							   cursor_text_mask_bits,
+							   (gchar *)cursor_text_mask_bits,
 							   cursor_text_mask_width,
 							   cursor_text_mask_height);
 		cursor =
@@ -517,13 +530,14 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 				   GdkEvent    *event,
 				   glView      *view)
 {
-	static gdouble      x0, y0;
-	static gboolean     dragging = FALSE;
+	static gdouble       x0, y0;
+	static gboolean      dragging = FALSE;
 	static glViewObject *view_text;
 	static GObject      *object;
-	gdouble             x, y;
+	gdouble              x, y;
 	GList               *lines;
 	gchar               *family;
+	glColorNode         *color_node;
 
 	gl_debug (DEBUG_VIEW, "");
 
@@ -533,6 +547,7 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 		gl_debug (DEBUG_VIEW, "BUTTON_PRESS");
 		switch (event->button.button) {
 		case 1:
+			color_node = gl_color_node_new_default ();
 			dragging = TRUE;
 			gnome_canvas_item_grab (canvas->root,
 						GDK_POINTER_MOTION_MASK |
@@ -553,8 +568,10 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 							 gl_view_get_default_font_weight (view));
 			gl_label_object_set_font_italic_flag (GL_LABEL_OBJECT(object),
 							      gl_view_get_default_font_italic_flag (view));
+								  
+			color_node->color = gl_color_set_opacity (gl_view_get_default_text_color (view), 0.5);
 			gl_label_object_set_text_color (GL_LABEL_OBJECT(object),
-							gl_color_set_opacity (gl_view_get_default_text_color (view), 0.5));
+							color_node);
 			gl_label_object_set_text_alignment (GL_LABEL_OBJECT(object),
 							    gl_view_get_default_text_alignment (view));
 			gl_label_object_set_text_line_spacing (GL_LABEL_OBJECT(object), gl_view_get_default_text_line_spacing (view));
@@ -566,6 +583,7 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 						      view);
 			x0 = x;
 			y0 = y;
+			gl_color_node_free (&color_node);
 			return TRUE;
 
 		default:
@@ -576,6 +594,7 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 		gl_debug (DEBUG_VIEW, "BUTTON_RELEASE");
 		switch (event->button.button) {
 		case 1:
+			color_node = gl_color_node_new_default ();
 			dragging = FALSE;
 			gnome_canvas_item_ungrab (canvas->root, event->button.time);
 			gnome_canvas_window_to_world (canvas,
@@ -591,8 +610,10 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 							 gl_view_get_default_font_weight (view));
 			gl_label_object_set_font_italic_flag (GL_LABEL_OBJECT(object),
 							      gl_view_get_default_font_italic_flag (view));
+			
+			color_node->color = gl_view_get_default_text_color (view);
 			gl_label_object_set_text_color (GL_LABEL_OBJECT(object),
-							gl_view_get_default_text_color (view));
+							color_node);
 			gl_label_object_set_text_alignment (GL_LABEL_OBJECT(object),
 							    gl_view_get_default_text_alignment (view));
 			gl_label_object_set_text_line_spacing (GL_LABEL_OBJECT(object), gl_view_get_default_text_line_spacing (view));
@@ -601,6 +622,7 @@ gl_view_text_create_event_handler (GnomeCanvas *canvas,
 			gl_view_unselect_all (view);
 			gl_view_object_select (GL_VIEW_OBJECT(view_text));
 			gl_view_arrow_mode (view);
+			gl_color_node_free (&color_node);
 			return TRUE;
 
 		default:
@@ -642,7 +664,7 @@ draw_hacktext (glViewText *view_text)
 	gboolean           font_italic_flag;
 	gdouble            font_size;
 	gdouble            text_line_spacing;
-	guint              color;
+	glColorNode       *color_node;
 	GtkJustification   just;
 	GnomeFont         *font;
 	GnomeGlyphList    *glyphlist;
@@ -663,7 +685,11 @@ draw_hacktext (glViewText *view_text)
 	text_line_spacing = gl_label_object_get_text_line_spacing (object);
 	font_weight = gl_label_object_get_font_weight (object);
 	font_italic_flag = gl_label_object_get_font_italic_flag (object);
-	color = gl_label_object_get_text_color (object);
+	color_node = gl_label_object_get_text_color (object);
+	if (color_node->field_flag)
+	{
+		color_node->color = GL_COLOR_MERGE_DEFAULT;
+	}
 	just = gl_label_object_get_text_alignment (object);
 	buffer = gl_label_text_get_buffer(GL_LABEL_TEXT(object));
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
@@ -681,7 +707,7 @@ draw_hacktext (glViewText *view_text)
 	gl_debug (DEBUG_VIEW, "2");
 
 	/* get Gnome Font */
-	font = gnome_font_find_closest_from_weight_slant (font_family,
+	font = gnome_font_find_closest_from_weight_slant ((guchar *)font_family,
 							  font_weight,
 							  font_italic_flag,
 							  font_size);
@@ -690,9 +716,9 @@ draw_hacktext (glViewText *view_text)
 	/* render to group, one item per line. */
 	for (i = 0; line[i] != NULL; i++) {
 
-		glyphlist = gnome_glyphlist_from_text_dumb (font, color,
+		glyphlist = gnome_glyphlist_from_text_dumb (font, color_node->color,
 							    0.0, 0.0,
-							    line[i]);
+							    (guchar *)line[i]);
 
 		gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 		w = bbox.x1;
@@ -734,12 +760,15 @@ draw_hacktext (glViewText *view_text)
 	}
 
 	/* clean up */
+	g_free (font_family);
+	gl_color_node_free (&color_node);
 	g_strfreev (line);
 	g_free (text);
 
 	gl_debug (DEBUG_VIEW, "END");
 }
 
+#ifdef EDIT_TEXT_IN_PLACE
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Draw cursor to item (group).                                   */
 /*--------------------------------------------------------------------------*/
@@ -755,7 +784,7 @@ draw_cursor (glViewText *view_text)
 	GnomeFontWeight    font_weight;
 	gboolean           font_italic_flag;
 	gdouble            font_size;
-	guint              color;
+	glColorNode       *color_node;
 	GtkJustification   just;
 	GnomeFont         *font;
 	GnomeGlyphList    *glyphlist;
@@ -779,7 +808,7 @@ draw_cursor (glViewText *view_text)
 	font_size = gl_label_object_get_font_size (object);
 	font_weight = gl_label_object_get_font_weight (object);
 	font_italic_flag = gl_label_object_get_font_italic_flag (object);
-	color = gl_label_object_get_text_color (object);
+	color_node = gl_label_object_get_text_color (object);
 	just = gl_label_object_get_text_alignment (object);
 	buffer = gl_label_text_get_buffer(GL_LABEL_TEXT(object));
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
@@ -798,7 +827,7 @@ draw_cursor (glViewText *view_text)
 	selection_flag = !gtk_text_iter_equal (&cursor_iter, &bound_iter);
 
 	/* get Gnome Font */
-	font = gnome_font_find_closest_from_weight_slant (font_family,
+	font = gnome_font_find_closest_from_weight_slant ((guchar *)font_family,
 							  font_weight,
 							  font_italic_flag,
 							  font_size);
@@ -809,9 +838,9 @@ draw_cursor (glViewText *view_text)
 		if ( i == cursor_line ) {
 			GnomeCanvasPoints *points;
 			
-			glyphlist = gnome_glyphlist_from_text_dumb (font, color,
+			glyphlist = gnome_glyphlist_from_text_dumb (font, color_node->color,
 								    0.0, 0.0,
-								    line[i]);
+								    (guchar *)line[i]);
 
 			gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 			gnome_glyphlist_unref (glyphlist);
@@ -832,9 +861,9 @@ draw_cursor (glViewText *view_text)
 				break;	/* shouldn't happen */
 			}
 
-			glyphlist = gnome_glyphlist_from_text_sized_dumb (font, color,
+			glyphlist = gnome_glyphlist_from_text_sized_dumb (font, color_node->color,
 									  0.0, 0.0,
-									  line[i],
+									  (guchar *)line[i],
 									  cursor_char);
 			gnome_glyphlist_bbox (glyphlist, affine, 0, &bbox);
 			gnome_glyphlist_unref (glyphlist);
@@ -867,6 +896,7 @@ draw_cursor (glViewText *view_text)
 	}
 
 	/* clean up */
+	gl_color_node_free (&color_node);
 	g_strfreev (line);
 	g_free (text);
 
@@ -950,6 +980,8 @@ blink_cb (glViewText *view_text)
 	return FALSE;
 }
 
+#endif /* EDIT_TEXT_IN_PLACE */
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Item event callback.                                           */
 /*--------------------------------------------------------------------------*/
@@ -960,4 +992,3 @@ item_event_cb (GnomeCanvasItem *item,
 {
 	gl_debug (DEBUG_VIEW, "");
 }
-
