@@ -23,7 +23,6 @@
 #ifndef __WDGT_MEDIA_SELECT_H__
 #define __WDGT_MEDIA_SELECT_H__
 
-#include <gnome.h>
 #include "hig.h"
 
 G_BEGIN_DECLS
@@ -44,9 +43,8 @@ typedef struct _glWdgtMediaSelectClass glWdgtMediaSelectClass;
 struct _glWdgtMediaSelect {
 	glHigVBox  parent_widget;
 
-	GtkWidget *page_size_entry;
+	GtkWidget *page_size_combo;
 	GtkWidget *template_combo;
-	GtkWidget *template_entry;
 
 	GtkWidget *mini_preview;
 
