@@ -1,3 +1,5 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+
 /*
  *  (GLABELS) Label and Business Card Creation program for GNOME
  *
@@ -22,8 +24,17 @@
 
 #include <config.h>
 
-#include "ui.h"
 #include "window.h"
+
+#include <glib/gi18n.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkstatusbar.h>
+
+#include "ui.h"
+#include "ui-commands.h"
 #include "util.h"
 #include "xml-label.h"
 #include "prefs.h"
@@ -36,7 +47,7 @@
 /*============================================================================*/
 
 #define DEFAULT_WINDOW_WIDTH  788
-#define DEFAULT_WINDOW_HEIGHT 560
+#define DEFAULT_WINDOW_HEIGHT 600
 
 #define CURSOR_INFO_WIDTH     150
 #define ZOOM_INFO_WIDTH        50
@@ -44,7 +55,7 @@
 /*============================================================================*/
 /* Private globals                                                            */
 /*============================================================================*/
-static BonoboWindowClass *parent_class;
+static GtkWindowClass *parent_class;
 
 static GList *window_list = NULL;
 
@@ -68,6 +79,11 @@ static gboolean window_delete_event_cb (glWindow      *window,
 static void     selection_changed_cb   (glView        *view,
 					glWindow      *window);
 
+static void   context_menu_activate_cb (glView       *view,
+					gint          button,
+					guint32       activate_time,
+					glWindow     *window);
+
 static void     zoom_changed_cb        (glView        *view,
 					gdouble        zoom,
 					glWindow      *window);
@@ -109,7 +125,7 @@ gl_window_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (BONOBO_TYPE_WINDOW,
+		type = g_type_register_static (GTK_TYPE_WINDOW,
 					       "glWindow", &info, 0);
 	}
 
@@ -136,16 +152,52 @@ gl_window_class_init (glWindowClass *class)
 static void
 gl_window_init (glWindow *window)
 {
-	BonoboUIContainer *ui_container;
-	BonoboUIComponent *ui_component;
+	GtkWidget        *vbox1;
+	GtkUIManager     *ui;
+	GtkWidget        *status_hbox;
 
 	gl_debug (DEBUG_WINDOW, "START");
 
-	ui_container = bonobo_window_get_ui_container(BONOBO_WINDOW(window));
-	ui_component = bonobo_ui_component_new_default ();
-	bonobo_ui_component_set_container (ui_component,
-					   BONOBO_OBJREF (ui_container),
-					   NULL);
+	vbox1 = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (window), vbox1);
+
+	window->ui = ui = gl_ui_new (window);
+	gtk_box_pack_start (GTK_BOX (vbox1),
+			    gtk_ui_manager_get_widget (ui, "/MenuBar"),
+			    FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1),
+			    gtk_ui_manager_get_widget (ui, "/MainToolBar"),
+			    FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1),
+			    gtk_ui_manager_get_widget (ui, "/DrawingToolBar"),
+			    FALSE, FALSE, 0);
+
+	window->hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1), window->hbox, TRUE, TRUE, 0);
+
+	window->sidebar = GL_UI_SIDEBAR (gl_ui_sidebar_new ());
+	gtk_box_pack_end (GTK_BOX (window->hbox), GTK_WIDGET (window->sidebar), FALSE, FALSE, 0);
+
+	window->property_bar = GL_UI_PROPERTY_BAR (gl_ui_property_bar_new ());
+	gtk_box_pack_start (GTK_BOX (vbox1), GTK_WIDGET (window->property_bar), FALSE, FALSE, 0);
+
+	status_hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1), status_hbox, FALSE, FALSE, 0);
+
+	window->status_bar = gtk_statusbar_new ();
+	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->status_bar), FALSE);
+	gtk_box_pack_start (GTK_BOX (status_hbox),
+			    window->status_bar,
+			    TRUE, TRUE, 0);
+	window->zoom_info = gtk_label_new (NULL);
+	gtk_widget_set_size_request (window->zoom_info, ZOOM_INFO_WIDTH, -1);
+	window->zoom_info_frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME(window->zoom_info_frame), GTK_SHADOW_IN);
+	gtk_container_add (GTK_CONTAINER(window->zoom_info_frame), window->zoom_info);
+	gtk_widget_show_all (window->zoom_info_frame);
+	gtk_box_pack_end (GTK_BOX (status_hbox),
+			  window->zoom_info_frame,
+			  FALSE, FALSE, 0);
 
 	window->cursor_info = gtk_label_new (NULL);
 	gtk_widget_set_size_request (window->cursor_info, CURSOR_INFO_WIDTH, -1);
@@ -153,23 +205,9 @@ gl_window_init (glWindow *window)
 	gtk_frame_set_shadow_type (GTK_FRAME(window->cursor_info_frame), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER(window->cursor_info_frame), window->cursor_info);
 	gtk_widget_show_all (window->cursor_info_frame);
-
-	window->zoom_info = gtk_label_new (NULL);
-	gtk_widget_set_size_request (window->zoom_info, ZOOM_INFO_WIDTH, -1);
-	window->zoom_info_frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME(window->zoom_info_frame), GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER(window->zoom_info_frame), window->zoom_info);
-	gtk_widget_show_all (window->zoom_info_frame);
-
-	gl_ui_init (ui_component,
-		    BONOBO_WINDOW (window),
-		    window->cursor_info_frame,
-		    window->zoom_info_frame);
-
-	window->property_bar =
-		GL_UI_PROPERTY_BAR(gl_ui_property_bar_new (ui_component));
-	window->sidebar =
-		GL_UI_SIDEBAR(gl_ui_sidebar_new (ui_component));
+	gtk_box_pack_end (GTK_BOX (status_hbox),
+			  window->cursor_info_frame,
+			  FALSE, FALSE, 0);
 
 	gtk_window_set_default_size (GTK_WINDOW (window),
 				     DEFAULT_WINDOW_WIDTH,
@@ -178,7 +216,15 @@ gl_window_init (glWindow *window)
 	g_signal_connect (G_OBJECT(window), "delete-event",
 			  G_CALLBACK(window_delete_event_cb), NULL);
 	
-	window->uic  = ui_component;
+	window->menu_tips_context_id =
+		gtk_statusbar_get_context_id (GTK_STATUSBAR (window->status_bar), "menu_tips");
+
+	window->print_dialog = NULL;
+	window->merge_dialog = NULL;
+	window->context_menu = GTK_MENU (gtk_ui_manager_get_widget (ui, "/ContextMenu"));
+	window->empty_selection_context_menu =
+		GTK_MENU (gtk_ui_manager_get_widget (ui, "/EmptySelectionContextMenu"));
+
 	window->view = NULL;
 
 	window_list = g_list_append (window_list, window);
@@ -216,9 +262,9 @@ gl_window_destroy (GtkObject *gtk_object)
 	window = GL_WINDOW (gtk_object);
 	window_list = g_list_remove (window_list, window);
 
-        if (window->uic) {
-		gl_ui_unref(window->uic);
-		window->uic = NULL;
+        if (window->ui) {
+		gl_ui_unref(window->ui);
+		window->ui = NULL;
         }
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy) {
@@ -230,7 +276,7 @@ gl_window_destroy (GtkObject *gtk_object)
 
 
 /****************************************************************************/
-/* Create an app window.                                                    */
+/** Create a glabels window.                                                */
 /****************************************************************************/
 GtkWidget *
 gl_window_new (void)
@@ -239,8 +285,7 @@ gl_window_new (void)
 
 	gl_debug (DEBUG_WINDOW, "START");
 
-	window = g_object_new (gl_window_get_type (),
-			       "win_name", "glabels",
+	window = g_object_new (GL_TYPE_WINDOW,
 			       "title",    _("(none) - gLabels"),
 			       NULL);
 
@@ -253,7 +298,7 @@ gl_window_new (void)
 }
 
 /****************************************************************************/
-/* Create an app window from a label.                                       */
+/** Create a glabels window from a label.                                   */
 /****************************************************************************/
 GtkWidget*
 gl_window_new_from_label (glLabel *label)
@@ -272,7 +317,7 @@ gl_window_new_from_label (glLabel *label)
 }
 
 /****************************************************************************/
-/* Create an app window from a file.                                        */
+/** Create a glabels window from a glabels file.                            */
 /****************************************************************************/
 GtkWidget*
 gl_window_new_from_file (const gchar *filename)
@@ -298,7 +343,7 @@ gl_window_new_from_file (const gchar *filename)
 }
 
 /****************************************************************************/
-/* Is window empty?                                                         */
+/** Is window empty?                                                        */
 /****************************************************************************/
 gboolean
 gl_window_is_empty (glWindow    *window)
@@ -310,7 +355,7 @@ gl_window_is_empty (glWindow    *window)
 }
 
 /****************************************************************************/
-/* Create view from label and place in window.                              */
+/** Create view from label and place in window.                             */
 /****************************************************************************/
 void
 gl_window_set_label (glWindow    *window,
@@ -333,7 +378,7 @@ gl_window_set_label (glWindow    *window,
 	}
 
 	window->view = gl_view_new (label);
-	bonobo_window_set_contents (BONOBO_WINDOW(window), window->view);
+	gtk_box_pack_start (GTK_BOX (window->hbox), window->view,TRUE, TRUE, 0);
 
 	gtk_widget_show_all (window->view);
 
@@ -351,7 +396,7 @@ gl_window_set_label (glWindow    *window,
 		gl_view_hide_markup (GL_VIEW(window->view));
 	}
 
-	gl_ui_update_all (window->uic, GL_VIEW(window->view));
+	gl_ui_update_all (window->ui, GL_VIEW(window->view));
 
 	gl_ui_property_bar_set_view (window->property_bar, GL_VIEW(window->view));
 	gl_ui_sidebar_set_view (window->sidebar, GL_VIEW(window->view));
@@ -364,6 +409,9 @@ gl_window_set_label (glWindow    *window,
 	g_signal_connect (G_OBJECT(window->view), "selection_changed",
 			  G_CALLBACK(selection_changed_cb), window);
 
+	g_signal_connect (G_OBJECT(window->view), "context_menu_activate",
+			  G_CALLBACK(context_menu_activate_cb), window);
+
 	g_signal_connect (G_OBJECT(window->view), "zoom_changed",
 			  G_CALLBACK(zoom_changed_cb), window);
 
@@ -383,7 +431,7 @@ gl_window_set_label (glWindow    *window,
 }
 
 /****************************************************************************/
-/* Return list of app windows.                                              */
+/** Return list of glabels windows.                                         */
 /****************************************************************************/
 const GList *
 gl_window_get_window_list (void)
@@ -393,7 +441,7 @@ gl_window_get_window_list (void)
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  Set window title based on name and state of label.              */
+/** PRIVATE.  Set window title based on name and state of label.             */
 /*---------------------------------------------------------------------------*/
 static void 
 set_window_title (glWindow *window,
@@ -425,7 +473,7 @@ set_window_title (glWindow *window,
 }
 
 /*-------------------------------------------------------------------------*/
-/* PRIVATE.  Window "delete-event" callback.                               */
+/** PRIVATE.  Window "delete-event" callback.                              */
 /*-------------------------------------------------------------------------*/
 static gboolean
 window_delete_event_cb (glWindow      *window,
@@ -444,7 +492,7 @@ window_delete_event_cb (glWindow      *window,
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  View "selection state changed" callback.                        */
+/** PRIVATE.  View "selection state changed" callback.                       */
 /*---------------------------------------------------------------------------*/
 static void 
 selection_changed_cb (glView   *view,
@@ -455,13 +503,42 @@ selection_changed_cb (glView   *view,
 	g_return_if_fail (view && GL_IS_VIEW (view));
 	g_return_if_fail (window && GL_IS_WINDOW (window));
 
-	gl_ui_update_selection_verbs (window->uic, view);
+	gl_ui_update_selection_verbs (window->ui, view);
 
 	gl_debug (DEBUG_WINDOW, "END");
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  View "zoom state changed" callback.                             */
+/** PRIVATE.  View "context menu activate" callback.                         */
+/*---------------------------------------------------------------------------*/
+static void
+context_menu_activate_cb (glView       *view,
+			  gint          button,
+			  guint32       activate_time,
+			  glWindow     *window)
+{
+        gl_debug (DEBUG_WINDOW, "START");
+
+        g_return_if_fail (view && GL_IS_VIEW (view));
+	g_return_if_fail (window && GL_IS_WINDOW (window));
+
+        if (gl_view_is_selection_empty (view)) {
+
+		gtk_menu_popup (GTK_MENU (window->empty_selection_context_menu),
+				NULL, NULL, NULL, NULL, button, activate_time);
+
+        } else {
+
+		gtk_menu_popup (GTK_MENU (window->context_menu),
+				NULL, NULL, NULL, NULL, button, activate_time);
+
+        }
+
+        gl_debug (DEBUG_WINDOW, "END");
+}
+
+/*---------------------------------------------------------------------------*/
+/** PRIVATE.  View "zoom state changed" callback.                            */
 /*---------------------------------------------------------------------------*/
 static void 
 zoom_changed_cb (glView   *view,
@@ -479,13 +556,13 @@ zoom_changed_cb (glView   *view,
 	gtk_label_set_text (GTK_LABEL(window->zoom_info), string);
 	g_free (string);
 
-	gl_ui_update_zoom_verbs (window->uic, view);
+	gl_ui_update_zoom_verbs (window->ui, view);
 
 	gl_debug (DEBUG_WINDOW, "END");
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  View "pointer moved" callback.                                  */
+/** PRIVATE.  View "pointer moved" callback.                                 */
 /*---------------------------------------------------------------------------*/
 static void
 pointer_moved_cb (glView   *view,
@@ -515,7 +592,7 @@ pointer_moved_cb (glView   *view,
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  View "pointer exit" callback.                                   */
+/** PRIVATE.  View "pointer exit" callback.                                  */
 /*---------------------------------------------------------------------------*/
 static void
 pointer_exit_cb (glView   *view,
@@ -532,7 +609,7 @@ pointer_exit_cb (glView   *view,
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  Label "name changed" callback.                                  */
+/** PRIVATE.  Label "name changed" callback.                                 */
 /*---------------------------------------------------------------------------*/
 static void 
 name_changed_cb (glLabel  *label,
@@ -549,7 +626,7 @@ name_changed_cb (glLabel  *label,
 }
 
 /*---------------------------------------------------------------------------*/
-/* PRIVATE.  Label "modified state changed" callback.                        */
+/** PRIVATE.  Label "modified state changed" callback.                       */
 /*---------------------------------------------------------------------------*/
 static void 
 modified_changed_cb (glLabel  *label,
@@ -562,7 +639,7 @@ modified_changed_cb (glLabel  *label,
 
 	set_window_title (window, label);
 
-	gl_ui_update_modified_verbs (window->uic, label);
+	gl_ui_update_modified_verbs (window->ui, label);
 
 	gl_debug (DEBUG_WINDOW, "END");
 }
