@@ -22,8 +22,9 @@
 
 #include <config.h>
 
-#include <gnome.h>
+#include "xml-label.h"
 
+#include <glib/gi18n.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
@@ -37,7 +38,6 @@
 #include "label-image.h"
 #include "label-barcode.h"
 #include "base64.h"
-#include "xml-label.h"
 #include "xml-label-04.h"
 #include "xml-label-191.h"
 #include <libglabels/template.h>
@@ -173,7 +173,7 @@ gl_xml_label_open (const gchar      *utf8_filename,
 
 	doc = xmlParseFile (filename);
 	if (!doc) {
-		g_warning (_("xmlParseFile error"));
+		g_message (_("xmlParseFile error"));
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
 		return NULL;
 	}
@@ -210,7 +210,7 @@ gl_xml_label_open_buffer (const gchar      *buffer,
 
 	doc = xmlParseDoc ((xmlChar *) buffer);
 	if (!doc) {
-		g_warning (_("xmlParseFile error"));
+		g_message (_("xmlParseFile error"));
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
 		return NULL;
 	}
@@ -247,39 +247,39 @@ xml_doc_to_label (xmlDocPtr         doc,
 
 	root = xmlDocGetRootElement (doc);
 	if (!root || !root->name) {
-		g_warning (_("No document root"));
+		g_message (_("No document root"));
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
 		return NULL;
 	}
 
-	ns = xmlSearchNsByHref (doc, root, GL_XML_NAME_SPACE);
+	ns = xmlSearchNsByHref (doc, root, (xmlChar *)GL_XML_NAME_SPACE);
 	if (ns != NULL) {
 		label = xml_parse_label (root, status);
 		if (label)
 			gl_label_set_compression (label, xmlGetDocCompressMode (doc));
 	} else {
 		/* Try compatability mode 0.1 */
-		ns = xmlSearchNsByHref (doc, root, COMPAT01_NAME_SPACE);
+		ns = xmlSearchNsByHref (doc, root, (xmlChar *)COMPAT01_NAME_SPACE);
 		if (ns != NULL)	{
-			g_warning (_("Importing from glabels 0.1 format"));
-			g_warning ("TODO");
+			g_message (_("Importing from glabels 0.1 format"));
+			g_message ("TODO");
 			label = NULL; /* TODO */
 		} else {
 			/* Try compatability mode 0.4 */
 			ns = xmlSearchNsByHref (doc, root,
-						COMPAT04_NAME_SPACE);
+						(xmlChar *)COMPAT04_NAME_SPACE);
 			if (ns != NULL)	{
-				g_warning (_("Importing from glabels 0.4 format"));
+				g_message (_("Importing from glabels 0.4 format"));
 				label = gl_xml_label_04_parse (root, status);
 			} else {
 				/* Try compatability mode 1.91 */
 				ns = xmlSearchNsByHref (doc, root,
-							COMPAT191_NAME_SPACE);
+							(xmlChar *)COMPAT191_NAME_SPACE);
 				if (ns != NULL)	{
-					g_warning (_("Importing from glabels 1.91 format"));
+					g_message (_("Importing from glabels 1.91 format"));
 					label = gl_xml_label_191_parse (root, status);
 				} else {
-					g_warning (_("bad document, unknown glabels Namespace"));
+					g_message (_("bad document, unknown glabels Namespace"));
 					*status = XML_LABEL_ERROR_OPEN_PARSE;
 					return NULL;
 				}
@@ -307,8 +307,8 @@ xml_parse_label (xmlNodePtr        root,
 
 	*status = XML_LABEL_OK;
 
-	if (!xmlStrEqual (root->name, "Glabels-document")) {
-		g_warning (_("Bad root node = \"%s\""), root->name);
+	if (!gl_xml_is_node (root, "Glabels-document")) {
+		g_message (_("Bad root node = \"%s\""), root->name);
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
 		return NULL;
 	}
@@ -317,7 +317,7 @@ xml_parse_label (xmlNodePtr        root,
 
 	/* Pass 1, extract data nodes to pre-load cache. */
 	for (child_node = root->xmlChildrenNode; child_node != NULL; child_node = child_node->next) {
-		if (xmlStrEqual (child_node->name, "Data")) {
+		if (gl_xml_is_node (child_node, "Data")) {
 			xml_parse_data (child_node, label);
 		}
 	}
@@ -327,7 +327,7 @@ xml_parse_label (xmlNodePtr        root,
              child_node != NULL;
 	     child_node = child_node->next) {
 
-		if (xmlStrEqual (child_node->name, "Template")) {
+		if (gl_xml_is_node (child_node, "Template")) {
 			template = gl_xml_template_parse_template_node (child_node);
 			if (!template) {
 				g_object_unref (label);
@@ -337,15 +337,15 @@ xml_parse_label (xmlNodePtr        root,
 			gl_template_register (template);
 			gl_label_set_template (label, template);
 			gl_template_free (template);
-		} else if (xmlStrEqual (child_node->name, "Objects")) {
+		} else if (gl_xml_is_node (child_node, "Objects")) {
 			xml_parse_objects (child_node, label);
-		} else if (xmlStrEqual (child_node->name, "Merge")) {
+		} else if (gl_xml_is_node (child_node, "Merge")) {
 			xml_parse_merge_fields (child_node, label);
-		} else if (xmlStrEqual (child_node->name, "Data")) {
+		} else if (gl_xml_is_node (child_node, "Data")) {
 			/* Handled in pass 1. */
 		} else {
 			if (!xmlNodeIsText (child_node)) {
-				g_warning (_("bad node in Document node =  \"%s\""),
+				g_message (_("bad node in Document node =  \"%s\""),
 					   child_node->name);
 				g_object_unref (label);
 				*status = XML_LABEL_ERROR_OPEN_PARSE;
@@ -376,21 +376,21 @@ xml_parse_objects (xmlNodePtr  node,
 
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
 
-		if (xmlStrEqual (child->name, "Object-text")) {
+		if (gl_xml_is_node (child, "Object-text")) {
 			xml_parse_object_text (child, label);
-		} else if (xmlStrEqual (child->name, "Object-box")) {
+		} else if (gl_xml_is_node (child, "Object-box")) {
 			xml_parse_object_box (child, label);
-		} else if (xmlStrEqual (child->name, "Object-ellipse")) {
+		} else if (gl_xml_is_node (child, "Object-ellipse")) {
 			xml_parse_object_ellipse (child, label);
-		} else if (xmlStrEqual (child->name, "Object-line")) {
+		} else if (gl_xml_is_node (child, "Object-line")) {
 			xml_parse_object_line (child, label);
-		} else if (xmlStrEqual (child->name, "Object-image")) {
+		} else if (gl_xml_is_node (child, "Object-image")) {
 			xml_parse_object_image (child, label);
-		} else if (xmlStrEqual (child->name, "Object-barcode")) {
+		} else if (gl_xml_is_node (child, "Object-barcode")) {
 			xml_parse_object_barcode (child, label);
 		} else {
 			if (!xmlNodeIsText (child)) {
-				g_warning (_("bad node =  \"%s\""), child->name);
+				g_message (_("bad node =  \"%s\""), child->name);
 				break;
 			}
 		}
@@ -430,7 +430,7 @@ xml_parse_object_text (xmlNodePtr  node,
 	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
 
 	/* justify attr */
-	string = xmlGetProp (node, "justify");
+	string = gl_xml_get_prop_string (node, "justify", NULL);
 	just = gl_util_string_to_just (string);
 	g_free (string);
 	gl_label_object_set_text_alignment (GL_LABEL_OBJECT(object), just);
@@ -450,12 +450,12 @@ xml_parse_object_text (xmlNodePtr  node,
 
 	/* Process children */
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
-		if (xmlStrEqual (child->name, "Span")) {
+		if (gl_xml_is_node (child, "Span")) {
 			xml_parse_toplevel_span (child, GL_LABEL_OBJECT(object));
 			break;
 		} else {
 			if (!xmlNodeIsText (child)) {
-				g_warning ("Unexpected Object-text child: \"%s\"",
+				g_message ("Unexpected Object-text child: \"%s\"",
 					   child->name);
 			}
 		}
@@ -471,12 +471,14 @@ static void
 xml_parse_object_box (xmlNodePtr  node,
 		      glLabel    *label)
 {
-	GObject *object;
-	gdouble  x, y;
-	gdouble  w, h;
-	gdouble  line_width;
-	guint    line_color, fill_color;
-	gdouble  affine[6];
+	GObject      *object;
+	gdouble       x, y;
+	gdouble       w, h;
+	gdouble       line_width;
+	glColorNode  *line_color_node;
+	gdouble       affine[6];
+	gchar        *string;
+	glColorNode  *fill_color_node;
 
 	gl_debug (DEBUG_XML, "START");
 
@@ -494,14 +496,32 @@ xml_parse_object_box (xmlNodePtr  node,
 
 	/* line attrs */
 	line_width = gl_xml_get_prop_length (node, "line_width", 1.0);
-	line_color = gl_xml_get_prop_uint (node, "line_color", 0);
 	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color);
+	
+	line_color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "line_color_field", NULL);
+	if ( string ) {
+		line_color_node->field_flag = TRUE;
+		line_color_node->key = string;
+	} else {
+		line_color_node->color = gl_xml_get_prop_uint (node, "line_color", 0);
+	}
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_color_node_free (&line_color_node);
+
 
 	/* fill attrs */
-	fill_color = gl_xml_get_prop_uint (node, "fill_color", 0);
-	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color);
-
+	fill_color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "fill_color_field", NULL);
+	if ( string ) {
+		fill_color_node->field_flag = TRUE;
+		fill_color_node->key = string;
+	} else {
+		fill_color_node->color = gl_xml_get_prop_uint (node, "fill_color", 0);
+	}
+	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node);
+	gl_color_node_free (&fill_color_node);
+	
 	/* affine attrs */
 	affine[0] = gl_xml_get_prop_double (node, "a0", 0.0);
 	affine[1] = gl_xml_get_prop_double (node, "a1", 0.0);
@@ -521,12 +541,14 @@ static void
 xml_parse_object_ellipse (xmlNodePtr  node,
 			  glLabel    *label)
 {
-	GObject *object;
-	gdouble  x, y;
-	gdouble  w, h;
-	gdouble  line_width;
-	guint    line_color, fill_color;
-	gdouble  affine[6];
+	GObject     *object;
+	gdouble      x, y;
+	gdouble      w, h;
+	gdouble      line_width;
+	glColorNode *line_color_node;
+	gdouble      affine[6];
+	gchar       *string;
+	glColorNode *fill_color_node;
 
 	gl_debug (DEBUG_XML, "START");
 
@@ -544,13 +566,31 @@ xml_parse_object_ellipse (xmlNodePtr  node,
 
 	/* line attrs */
 	line_width = gl_xml_get_prop_length (node, "line_width", 1.0);
-	line_color = gl_xml_get_prop_uint (node, "line_color", 0);
 	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color);
+
+	line_color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "line_color_field", NULL);
+	if ( string ) {
+		line_color_node->field_flag = TRUE;
+		line_color_node->key = string;
+	} else {
+		line_color_node->color = gl_xml_get_prop_uint (node, "line_color", 0);		
+	}
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_color_node_free (&line_color_node);
+
 
 	/* fill attrs */
-	fill_color = gl_xml_get_prop_uint (node, "fill_color", 0);
-	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color);
+	fill_color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "fill_color_field", NULL);
+	if ( string ) {
+		fill_color_node->field_flag = TRUE;
+		fill_color_node->key = string;
+	} else {
+		fill_color_node->color = gl_xml_get_prop_uint (node, "fill_color", 0);
+	}
+	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node);
+	gl_color_node_free (&fill_color_node);
 
 	/* affine attrs */
 	affine[0] = gl_xml_get_prop_double (node, "a0", 0.0);
@@ -571,12 +611,13 @@ static void
 xml_parse_object_line (xmlNodePtr  node,
 		       glLabel    *label)
 {
-	GObject *object;
-	gdouble  x, y;
-	gdouble  dx, dy;
-	gdouble  line_width;
-	guint    line_color;
-	gdouble  affine[6];
+	GObject     *object;
+	gdouble      x, y;
+	gdouble      dx, dy;
+	gdouble      line_width;
+	glColorNode *line_color_node;
+	gdouble      affine[6];
+	gchar       *string;
 
 	gl_debug (DEBUG_XML, "START");
 
@@ -594,9 +635,18 @@ xml_parse_object_line (xmlNodePtr  node,
 
 	/* line attrs */
 	line_width = gl_xml_get_prop_length (node, "line_width", 1.0);
-	line_color = gl_xml_get_prop_uint (node, "line_color", 0);
 	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color);
+	
+	line_color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "line_color_field", NULL);
+	if ( string ) {
+		line_color_node->field_flag = TRUE;
+		line_color_node->key = string;
+	} else {
+		line_color_node->color = gl_xml_get_prop_uint (node, "line_color", 0);		
+	}
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_color_node_free (&line_color_node);
 
 	/* affine attrs */
 	affine[0] = gl_xml_get_prop_double (node, "a0", 0.0);
@@ -634,23 +684,25 @@ xml_parse_object_image (xmlNodePtr  node,
 	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
 
 	/* src or field attr */
-	string = xmlGetProp (node, "src");
+	string = gl_xml_get_prop_string (node, "src", NULL);
 	if ( string ) {
 		filename = g_new0 (glTextNode, 1);
 		filename->field_flag = FALSE;
-		filename->data = string;
+		filename->data = g_strdup ((gchar *)string);
 		gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename);
 		gl_text_node_free (&filename);
+		xmlFree (string);
 	} else {
-		string = xmlGetProp (node, "field");
+		string = gl_xml_get_prop_string (node, "field", NULL);
 		if ( string ) {
 			filename = g_new0 (glTextNode, 1);
 			filename->field_flag = TRUE;
-			filename->data = string;
+			filename->data = g_strdup ((gchar *)string);
 			gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename);
 			gl_text_node_free (&filename);
+			xmlFree (string);
 		} else {
-			g_warning ("Missing Object-image src or field attr");
+			g_message ("Missing Object-image src or field attr");
 		}
 	}
 
@@ -686,7 +738,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 	gchar              *id;
 	gboolean            text_flag;
 	gboolean            checksum_flag;
-	guint               color;
+	glColorNode        *color_node;
 	guint               format_digits;
 	gdouble             affine[6];
 
@@ -705,18 +757,27 @@ xml_parse_object_barcode (xmlNodePtr  node,
 	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
 
 	/* prop attrs */
-	id = xmlGetProp (node, "style");
+	id = gl_xml_get_prop_string (node, "style", NULL);
 	text_flag = gl_xml_get_prop_boolean (node, "text", FALSE);
 	checksum_flag = gl_xml_get_prop_boolean (node, "checksum", TRUE);
-	color = gl_xml_get_prop_uint (node, "color", 0);
 	format_digits = gl_xml_get_prop_uint (node, "format", 10);
 	gl_label_barcode_set_props (GL_LABEL_BARCODE(object),
-				    id, text_flag, checksum_flag, format_digits);
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), color);
+				    (gchar *)id, text_flag, checksum_flag, format_digits);
 	g_free (id);
+	
+	color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "color_field", NULL);
+	if ( string ) {
+		color_node->field_flag = TRUE;
+		color_node->key = string;
+	} else {
+		color_node->color = gl_xml_get_prop_uint (node, "color", 0);		
+	}
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), color_node);
+	gl_color_node_free (&color_node);
 
 	/* data or field attr */
-	string = xmlGetProp (node, "data");
+	string = gl_xml_get_prop_string (node, "data", NULL);
 	if ( string ) {
 		text_node = g_new0 (glTextNode, 1);
 		text_node->field_flag = FALSE;
@@ -724,7 +785,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 		gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node);
 		gl_text_node_free (&text_node);
 	} else {
-		string = xmlGetProp (node, "field");
+		string = gl_xml_get_prop_string (node, "field", NULL);
 		if ( string ) {
 			text_node = g_new0 (glTextNode, 1);
 			text_node->field_flag = TRUE;
@@ -732,7 +793,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 			gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node);
 			gl_text_node_free (&text_node);
 		} else {
-			g_warning ("Missing Object-barcode data or field attr");
+			g_message ("Missing Object-barcode data or field attr");
 		}
 	}
 
@@ -761,11 +822,11 @@ xml_parse_merge_fields (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	string = xmlGetProp (node, "type");
+	string = gl_xml_get_prop_string (node, "type", NULL);
 	merge = gl_merge_new (string);
 	g_free (string);
 
-	string = xmlGetProp (node, "src");
+	string = gl_xml_get_prop_string (node, "src", NULL);
 	gl_merge_set_src (merge, string);
 	g_free (string);
 
@@ -789,11 +850,11 @@ xml_parse_data (xmlNodePtr  node,
 
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
 
-		if (xmlStrEqual (child->name, "Pixdata")) {
+		if (gl_xml_is_node (child, "Pixdata")) {
 			xml_parse_pixdata (child, label);
 		} else {
 			if (!xmlNodeIsText (child)) {
-				g_warning (_("bad node in Data node =  \"%s\""),
+				g_message (_("bad node in Data node =  \"%s\""),
 					   child->name);
 			}
 		}
@@ -819,10 +880,10 @@ xml_parse_pixdata (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	name = xmlGetProp (node, "name");
-	base64 = xmlNodeGetContent (node);
+	name = gl_xml_get_prop_string (node, "name", NULL);
+	base64 = gl_xml_get_node_content (node);
 
-	stream = gl_base64_decode (base64, &stream_length);
+	stream = gl_base64_decode ((gchar *)base64, &stream_length);
 	pixdata = g_new0 (GdkPixdata, 1);
 	ret = gdk_pixdata_deserialize (pixdata, stream_length, stream, NULL);
 
@@ -830,11 +891,12 @@ xml_parse_pixdata (xmlNodePtr  node,
 		pixbuf = gdk_pixbuf_from_pixdata (pixdata, TRUE, NULL);
 
 		pixbuf_cache = gl_label_get_pixbuf_cache (label);
-		gl_pixbuf_cache_add_pixbuf (pixbuf_cache, name, pixbuf);
+		gl_pixbuf_cache_add_pixbuf (pixbuf_cache, (gchar *)name, pixbuf);
 	}
 
 	g_free (name);
 	g_free (base64);
+
 	g_free (stream);
 	g_free (pixdata);
 
@@ -852,7 +914,7 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	gdouble           font_size;
 	GnomeFontWeight   font_weight;
 	gboolean          font_italic_flag;
-	guint             color;
+	glColorNode      *color_node;
 	gdouble           text_line_spacing;
 	gchar            *string;
 	GList            *lines, *text_nodes;
@@ -862,7 +924,7 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	gl_debug (DEBUG_XML, "START");
 
 	/* Font family attr */
-	font_family = xmlGetProp (node, "font_family");
+	font_family = gl_xml_get_prop_string (node, "font_family", "Sans");
 	gl_label_object_set_font_family (object, font_family);
 	g_free (font_family);
 
@@ -871,7 +933,7 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	gl_label_object_set_font_size (object, font_size);
 
 	/* Font weight attr */
-	string = xmlGetProp (node, "font_weight");
+	string = gl_xml_get_prop_string (node, "font_weight", NULL);
 	font_weight = gl_util_string_to_weight (string);
 	g_free (string);
 	gl_label_object_set_font_weight (object, font_weight);
@@ -881,8 +943,17 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	gl_label_object_set_font_italic_flag (object, font_italic_flag);
 
 	/* Text color attr */
-	color = gl_xml_get_prop_uint (node, "color", 0);
-	gl_label_object_set_text_color (object, color);
+	color_node = gl_color_node_new_default ();
+	string = gl_xml_get_prop_string (node, "color_field", NULL);
+	if ( string ) {
+		color_node->field_flag = TRUE;
+		color_node->key = string;
+	} else {
+		color_node->color = gl_xml_get_prop_uint (node, "color", 0);		
+	}
+	gl_label_object_set_text_color (object, color_node);
+	gl_color_node_free (&color_node);
+	
 
 	/* Text line spacing attr  */
 	text_line_spacing = gl_xml_get_prop_double (node, "line_spacing", 1.0);
@@ -894,39 +965,39 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
 
 		if (xmlNodeIsText (child)) {
-			gchar *data = xmlNodeGetContent (child); 
+			gchar *data = gl_xml_get_node_content (child); 
 
-			/* Hack: if the first char is LF, it's a xml formatting string */
+			/* Hack: if the first char is LF, it's an xml formatting string */
 			if (data[0] != '\n') { 
 				/* Literal text */
 				text_node = g_new0 (glTextNode, 1);
 				text_node->field_flag = FALSE;
-				text_node->data = data;
+				text_node->data = g_strdup ((gchar *)data);
 				text_nodes = g_list_append (text_nodes, text_node);
 			}
-			else 
-				g_free (data);
+			g_free (data);
 
-		} else if (xmlStrEqual (child->name, "Span")) {
+		} else if (gl_xml_is_node (child, "Span")) {
 
-			g_warning ("Unexpected rich text (not supported, yet!)");
+			g_message ("Unexpected rich text (not supported, yet!)");
 
-		} else if (xmlStrEqual (child->name, "Field")) {
+		} else if (gl_xml_is_node (child, "Field")) {
 
 			/* Field node */
+			string = gl_xml_get_prop_string (child, "name", NULL);
 			text_node = g_new0 (glTextNode, 1);
 			text_node->field_flag = TRUE;
-			text_node->data = xmlGetProp (child, "name");
+			text_node->data = string;
 			text_nodes = g_list_append (text_nodes, text_node);
 
-		} else if (xmlStrEqual (child->name, "NL")) {
+		} else if (gl_xml_is_node (child, "NL")) {
 
 			/* Store line. */
 			lines = g_list_append (lines, text_nodes);
 			text_nodes = NULL;
 
 		} else {
-			g_warning ("Unexpected Span child: \"%s\"", child->name);
+			g_message ("Unexpected Span child: \"%s\"", child->name);
 		}
 
 	}
@@ -959,14 +1030,14 @@ gl_xml_label_save (glLabel          *label,
 
 	filename = g_filename_from_utf8 (utf8_filename, -1, NULL, NULL, NULL);
 	if (!filename)
-		g_warning (_("Utf8 conversion error."));
+		g_message (_("Utf8 conversion error."));
 	else {
 		xmlSetDocCompressMode (doc, gl_label_get_compression (label));
 		xml_ret = xmlSaveFormatFile (filename, doc, TRUE);
 		xmlFreeDoc (doc);
 		if (xml_ret == -1) {
 
-			g_warning (_("Problem saving xml file."));
+			g_message (_("Problem saving xml file."));
 			*status = XML_LABEL_ERROR_SAVE_FILE;
 
 		} else {
@@ -1022,10 +1093,10 @@ xml_label_to_doc (glLabel          *label,
 
 	LIBXML_TEST_VERSION;
 
-	doc = xmlNewDoc ("1.0");
-	doc->xmlRootNode = xmlNewDocNode (doc, NULL, "Glabels-document", NULL);
+	doc = xmlNewDoc ((xmlChar *)"1.0");
+	doc->xmlRootNode = xmlNewDocNode (doc, NULL, (xmlChar *)"Glabels-document", NULL);
 
-	ns = xmlNewNs (doc->xmlRootNode, GL_XML_NAME_SPACE, NULL);
+	ns = xmlNewNs (doc->xmlRootNode, (xmlChar *)GL_XML_NAME_SPACE, NULL);
 	xmlSetNs (doc->xmlRootNode, ns);
 
 	template = gl_label_get_template (label);
@@ -1065,8 +1136,8 @@ xml_create_objects (xmlNodePtr  root,
 
 	rotate_flag = gl_label_get_rotate_flag (label);
 
-	node = xmlNewChild (root, ns, "Objects", NULL);
-	xmlSetProp (node, "id", "0");
+	node = xmlNewChild (root, ns, (xmlChar *)"Objects", NULL);
+	gl_xml_set_prop_string (node, "id", "0");
 	gl_xml_set_prop_boolean (node, "rotate", rotate_flag);
 
 	for (p = label->objects; p != NULL; p = p->next) {
@@ -1086,7 +1157,7 @@ xml_create_objects (xmlNodePtr  root,
 		} else if ( GL_IS_LABEL_BARCODE(object) ) {
 			xml_create_object_barcode (node, ns, object);
 		} else {
-			g_warning ("Unknown label object");
+			g_message ("Unknown label object");
 		}
 
 	}
@@ -1111,7 +1182,7 @@ xml_create_object_text (xmlNodePtr     root,
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-text", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-text", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1125,7 +1196,7 @@ xml_create_object_text (xmlNodePtr     root,
 
 	/* justify attr */
 	just = gl_label_object_get_text_alignment (object);
-	xmlSetProp (node, "justify", gl_util_just_to_string (just));
+	gl_xml_set_prop_string (node, "justify", gl_util_just_to_string (just));
 
 	/* auto_shrink attr */
 	auto_shrink = gl_label_text_get_auto_shrink (GL_LABEL_TEXT (object));
@@ -1158,12 +1229,13 @@ xml_create_object_box (xmlNodePtr     root,
 	gdouble           x, y;
 	gdouble           w, h;
 	gdouble           line_width;
-	guint             line_color, fill_color;
+	glColorNode      *line_color_node;
 	gdouble           affine[6];
+	glColorNode *fill_color_node;
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-box", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-box", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1177,13 +1249,30 @@ xml_create_object_box (xmlNodePtr     root,
 
 	/* line attrs */
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
 	gl_xml_set_prop_length (node, "line_width", line_width);
-	gl_xml_set_prop_uint_hex (node, "line_color", line_color);
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	if (line_color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "line_color_field", line_color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "line_color", line_color_node->color);
+	}
+	gl_color_node_free (&line_color_node);
 
-	/* fill attrs */
-	fill_color = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
-	gl_xml_set_prop_uint_hex (node, "fill_color", fill_color);
+	/* fill attrs (color or field) */
+	fill_color_node = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
+	if (fill_color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "fill_color_field", fill_color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "fill_color", fill_color_node->color);
+	}
+	gl_color_node_free (&fill_color_node);
 
 	/* affine attrs */
 	gl_label_object_get_affine (object, affine);
@@ -1209,12 +1298,13 @@ xml_create_object_ellipse (xmlNodePtr     root,
 	gdouble           x, y;
 	gdouble           w, h;
 	gdouble           line_width;
-	guint             line_color, fill_color;
+	glColorNode      *line_color_node;
 	gdouble           affine[6];
+	glColorNode *fill_color_node;
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-ellipse", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-ellipse", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1228,13 +1318,31 @@ xml_create_object_ellipse (xmlNodePtr     root,
 
 	/* line attrs */
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
 	gl_xml_set_prop_length (node, "line_width", line_width);
-	gl_xml_set_prop_uint_hex (node, "line_color", line_color);
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	if (line_color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "line_color_field", line_color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "line_color", line_color_node->color);
+	}
+	gl_color_node_free (&line_color_node);
 
-	/* fill attrs */
-	fill_color = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
-	gl_xml_set_prop_uint_hex (node, "fill_color", fill_color);
+
+	/* fill attrs (color or field) */
+	fill_color_node = gl_label_object_get_fill_color (GL_LABEL_OBJECT(object));
+	if (fill_color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "fill_color_field", fill_color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "fill_color", fill_color_node->color);
+	}
+	gl_color_node_free (&fill_color_node);
 
 	/* affine attrs */
 	gl_label_object_get_affine (object, affine);
@@ -1260,12 +1368,12 @@ xml_create_object_line (xmlNodePtr     root,
 	gdouble           x, y;
 	gdouble           dx, dy;
 	gdouble           line_width;
-	guint             line_color;
+	glColorNode      *line_color_node;
 	gdouble           affine[6];
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-line", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-line", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1279,9 +1387,19 @@ xml_create_object_line (xmlNodePtr     root,
 
 	/* line attrs */
 	line_width = gl_label_object_get_line_width (GL_LABEL_OBJECT(object));
-	line_color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
 	gl_xml_set_prop_length (node, "line_width", line_width);
-	gl_xml_set_prop_uint_hex (node, "line_color", line_color);
+	
+	line_color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	if (line_color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "line_color_field", line_color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "line_color", line_color_node->color);
+	}
+	gl_color_node_free (&line_color_node);
+
 
 	/* affine attrs */
 	gl_label_object_get_affine (object, affine);
@@ -1311,7 +1429,7 @@ xml_create_object_image (xmlNodePtr     root,
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-image", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-image", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1326,9 +1444,9 @@ xml_create_object_image (xmlNodePtr     root,
 	/* src OR field attr */
 	filename = gl_label_image_get_filename (GL_LABEL_IMAGE(object));
 	if (filename->field_flag) {
-		xmlSetProp (node, "field", filename->data);
+		gl_xml_set_prop_string (node, "field", filename->data);
 	} else {
-		xmlSetProp (node, "src", filename->data);
+		gl_xml_set_prop_string (node, "src", filename->data);
 	}
 	gl_text_node_free (&filename);
 
@@ -1359,13 +1477,13 @@ xml_create_object_barcode (xmlNodePtr     root,
 	gchar            *id;
 	gboolean          text_flag;
 	gboolean          checksum_flag;
-	guint             color;
+	glColorNode      *color_node;
 	guint             format_digits;
 	gdouble           affine[6];
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Object-barcode", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Object-barcode", NULL);
 
 	/* position attrs */
 	gl_label_object_get_position (object, &x, &y);
@@ -1380,20 +1498,31 @@ xml_create_object_barcode (xmlNodePtr     root,
 	/* Barcode properties attrs */
 	gl_label_barcode_get_props (GL_LABEL_BARCODE(object),
 				    &id, &text_flag, &checksum_flag, &format_digits);
-	color = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
-	xmlSetProp (node, "style", id);
+	gl_xml_set_prop_string (node, "style", id);
 	gl_xml_set_prop_boolean (node, "text", text_flag);
 	gl_xml_set_prop_boolean (node, "checksum", checksum_flag);
-	gl_xml_set_prop_uint_hex (node, "color", color);
+	
 	g_free (id);
+	
+	color_node = gl_label_object_get_line_color (GL_LABEL_OBJECT(object));
+	if (color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "color_field", color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "color", color_node->color);
+	}
+	gl_color_node_free (&color_node);
+
 
 	/* data OR field attr */
 	text_node = gl_label_barcode_get_data (GL_LABEL_BARCODE(object));
 	if (text_node->field_flag) {
-		xmlSetProp (node, "field", text_node->data);
+		gl_xml_set_prop_string (node, "field", text_node->data);
 	        gl_xml_set_prop_int (node, "format", format_digits);
 	} else {
-		xmlSetProp (node, "data", text_node->data);
+		gl_xml_set_prop_string (node, "data", text_node->data);
 	}
 	gl_text_node_free (&text_node);
 
@@ -1426,14 +1555,14 @@ xml_create_merge_fields (xmlNodePtr  root,
 
 	merge = gl_label_get_merge (label);
 
-	node = xmlNewChild (root, ns, "Merge", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Merge", NULL);
 
 	string = gl_merge_get_name (merge);
-	xmlSetProp (node, "type", string);
+	gl_xml_set_prop_string (node, "type", string);
 	g_free (string);
 
 	string = gl_merge_get_src (merge);
-	xmlSetProp (node, "src", string);
+	gl_xml_set_prop_string (node, "src", string);
 	g_free (string);
 
 	g_object_unref (G_OBJECT(merge));
@@ -1455,7 +1584,7 @@ xml_create_data (xmlNodePtr  root,
 
 	gl_debug (DEBUG_XML, "START");
 
-	node = xmlNewChild (root, ns, "Data", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Data", NULL);
 
 	pixbuf_cache = gl_label_get_pixbuf_cache (label);
 	name_list = gl_pixbuf_cache_get_name_list (pixbuf_cache);
@@ -1499,9 +1628,9 @@ xml_create_pixdata (xmlNodePtr  root,
 		stream = gdk_pixdata_serialize (pixdata, &stream_length);
 		base64 = gl_base64_encode (stream, stream_length);
 
-		node = xmlNewChild (root, ns, "Pixdata", base64);
-		xmlSetProp (node, "name", name);
-		xmlSetProp (node, "encoding", "Base64");
+		node = xmlNewChild (root, ns, (xmlChar *)"Pixdata", (xmlChar *)base64);
+		gl_xml_set_prop_string (node, "name", name);
+		gl_xml_set_prop_string (node, "encoding", "Base64");
 
 		gl_pixbuf_cache_remove_pixbuf (pixbuf_cache, name);
 
@@ -1527,14 +1656,14 @@ xml_create_toplevel_span (xmlNodePtr        root,
 	gdouble           font_size;
 	GnomeFontWeight   font_weight;
 	gboolean          font_italic_flag;
-	guint             color;
+	glColorNode      *color_node;
 	GtkJustification  just;
 	gdouble           text_line_spacing;
 	GList            *lines, *p_line, *p_node;
 	glTextNode       *text_node;
 	xmlNodePtr        child;
 
-	node = xmlNewChild (root, ns, "Span", NULL);
+	node = xmlNewChild (root, ns, (xmlChar *)"Span", NULL);
 
 	/* All span attrs at top level. */
 	font_family = gl_label_object_get_font_family (GL_LABEL_OBJECT(object_text));
@@ -1542,13 +1671,24 @@ xml_create_toplevel_span (xmlNodePtr        root,
 	text_line_spacing = gl_label_object_get_text_line_spacing (GL_LABEL_OBJECT(object_text));
 	font_weight = gl_label_object_get_font_weight (GL_LABEL_OBJECT(object_text));
 	font_italic_flag = gl_label_object_get_font_italic_flag (GL_LABEL_OBJECT(object_text));
-	color = gl_label_object_get_text_color (GL_LABEL_OBJECT(object_text));
+	
+	color_node = gl_label_object_get_text_color (GL_LABEL_OBJECT(object_text));
+	if (color_node->field_flag)
+	{
+		gl_xml_set_prop_string (node, "color_field", color_node->key);
+	}
+	else
+	{
+		gl_xml_set_prop_uint_hex (node, "color", color_node->color);
+	}
+	gl_color_node_free (&color_node);
+	
 	just = gl_label_object_get_text_alignment (GL_LABEL_OBJECT(object_text));
-	xmlSetProp (node, "font_family", font_family);
+	gl_xml_set_prop_string (node, "font_family", font_family);
 	gl_xml_set_prop_double (node, "font_size", font_size);
-	xmlSetProp (node, "font_weight", gl_util_weight_to_string (font_weight));
+	gl_xml_set_prop_string (node, "font_weight", gl_util_weight_to_string (font_weight));
 	gl_xml_set_prop_boolean (node, "font_italic", font_italic_flag);
-	gl_xml_set_prop_uint_hex (node, "color", color);
+	
 	gl_xml_set_prop_double (node, "line_spacing", text_line_spacing);
 
 	/* Build children. */
@@ -1560,16 +1700,16 @@ xml_create_toplevel_span (xmlNodePtr        root,
 			text_node = (glTextNode *) p_node->data;
 
 			if (text_node->field_flag) {
-				child = xmlNewChild (node, ns, "Field", NULL);
-				xmlSetProp (child, "name", text_node->data);
+				child = xmlNewChild (node, ns, (xmlChar *)"Field", NULL);
+				gl_xml_set_prop_string (child, "name", text_node->data);
 			} else {
-				xmlNodeAddContent (node, text_node->data);
+				xmlNodeAddContent (node, (xmlChar *)text_node->data);
 			}
 
 		}
 
 		if ( p_line->next ) {
-			child = xmlNewChild (node, ns, "NL", NULL);
+			child = xmlNewChild (node, ns, (xmlChar *)"NL", NULL);
 		}
 
 	}
